@@ -59,4 +59,5 @@ class TastyPieRequestTimingMiddleware(GraphiteRequestTimingMiddleware):
             request._view_name = view_kwargs['resource_name']
             request._start_time = time.time()
         except (AttributeError, KeyError):
-            pass
+            super(TastyPieRequestTimingMiddleware, self).process_view(request,
+                view_func, view_args, view_kwargs)
