@@ -47,27 +47,35 @@ class NoseStatsd(Plugin):
             counts.setdefault(k, [])
             [counts[k].append(_v) for _v in v]
 
-        write('=' * 70)
-        write('%s | Number | Avg (ms) | Total (ms)' %
-              ('Statsd Keys'.ljust(longest)))
-        write('-' * 70)
+        header = '%s | Number |  Avg (ms)  | Total (ms)' % (
+            'Statsd Keys'.ljust(longest))
+        header_len = len(header)
+
+        write('')
+        write('=' * header_len)
+        write('%s | Number |  Avg (ms)  | Total (ms)' % (
+            'Statsd Keys'.ljust(longest)))
+        write('-' * header_len)
         if not timings:
             write('None')
 
         for k in sorted(timings.keys()):
             v = timings[k]
-            write('%s | %s | %0.6f | %0.3f' %
-                  (k.ljust(longest), str(len(v)).rjust(6),
-                  sum(v) / float(len(v)), sum(v)))
+            write('%s | %s | %s | %s' % (
+                k.ljust(longest),
+                str(len(v)).rjust(6),
+                ('%0.5f' % (sum(v) / float(len(v)))).rjust(10),
+                ('%0.3f' % sum(v)).rjust(10)))
 
-        write('=' * 70)
+        write('=' * header_len)
         write('%s | Number | Total' % ('Statsd Counts'.ljust(longest)))
-        write('-' * 70)
+        write('-' * header_len)
         if not counts:
             write('None')
 
         for k in sorted(counts.keys()):
             v = counts[k]
-            write('%s | %s | %d' %
-                  (k.ljust(longest), str(len(v)).rjust(6),
-                   sum([x * y for x, y in v])))
+            write('%s | %s | %d' % (
+                k.ljust(longest),
+                str(len(v)).rjust(6),
+                sum([x * y for x, y in v])))
